package edu.bu.metcs673.trackr.controller;

import edu.bu.metcs673.trackr.api.GenericApiResponse;
import edu.bu.metcs673.trackr.common.CommonConstants;
import edu.bu.metcs673.trackr.common.TrackrInputValidationException;
import net.minidev.json.JSONObject;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.context.request.WebRequest;

import javax.validation.ConstraintViolationException;

/**
 * Used to handle the custom exceptions that are generated by the application.
 * Returns an error response to the user.
 *
 * @author Tim Flucker
 */
@ControllerAdvice
public class ExceptionController {

    /**
     * If a TrackrInputValidationException is thrown, then this block of code will
     * execute.
     *
     * @param exception
     * @param request
     * @return
     */
    @ExceptionHandler(value = {TrackrInputValidationException.class, HttpMessageNotReadableException.class})
    protected ResponseEntity<GenericApiResponse> handleCustomExceptions(Exception exception, WebRequest request) {
        return new ResponseEntity<GenericApiResponse>(GenericApiResponse.errorResponse(exception.getMessage()),
                HttpStatus.BAD_REQUEST);
    }

    /**
     * If a ConstraintViolationException or MethodArgumentNotValidException is
     * thrown, then this block of code will execute. All error messages that are
     * captured in the validation will be returned to the user as a Map string.
     *
     * @param exception
     * @param request
     * @return
     */
    @ExceptionHandler(value = {ConstraintViolationException.class, MethodArgumentNotValidException.class})
    protected ResponseEntity<GenericApiResponse> handleAnnotationValidationExceptions(MethodArgumentNotValidException exception,
                                                                                      WebRequest request) {
        JSONObject errorMap = new JSONObject();
        exception.getBindingResult().getAllErrors().forEach((error) -> {
            String fieldName = ((FieldError) error).getField();
            String errorMessage = error.getDefaultMessage();
            errorMap.put(fieldName, errorMessage);
        });
        return new ResponseEntity<GenericApiResponse>(GenericApiResponse.errorResponse(CommonConstants.VALIDATION_ERRORS, errorMap),
                HttpStatus.BAD_REQUEST);
    }

    @ExceptionHandler(value = {AuthenticationException.class, UsernameNotFoundException.class})
    protected ResponseEntity<GenericApiResponse> handleAuthenticationExceptions(Exception exception, WebRequest request) {
        exception.printStackTrace();
        return new ResponseEntity<GenericApiResponse>(GenericApiResponse.errorResponse(exception.getMessage()),
                HttpStatus.BAD_REQUEST);
    }

}
